<a name="readme-top"></a>
<!-- PROJECT LOGO -->
<br />
<div align="center">
  <a href="https://gitlab.eduxiji.net/202310358111361/project1466467-176097">
    <img src="/doc/image/logo.png" alt="Logo" width="300" height="200">
  </a>

  <h3 align="center">Loongarch-XV6</h3>

  <p align="center">
    An xv6 kernel ported to the Loongarch architecture
    <br />
    <a href="https://gitlab.eduxiji.net/202310358111361/project1466467-176097/-/tree/master/doc"><strong>Explore the docs »</strong></a>
    <br />
    <br />
    <a href="https://gitlab.eduxiji.net/202310358111361/project1466467-176097/-/tree/master/">View Demo</a>
    ·
    <a href="https://gitlab.eduxiji.net/202310358111361/project1466467-176097/-/issues">Report Bug</a>
    ·
    <a href="https://gitlab.eduxiji.net/202310358111361/project1466467-176097/-/issues">Request Feature</a>
  </p>
</div>
<!-- TABLE OF CONTENTS -->
<details>
  <summary>Table of Contents</summary>
  <ol>
    <li>
      <a href="#about-the-project">About The Project</a>
      <ul>
        <li><a href="#built-with">Built With</a></li>
      </ul>
    </li>
    <li><a href="#docs">Document</a></li>
    <li>
      <a href="#getting-started">Getting Started</a>
      <ul>
        <li><a href="#prerequisites">Prerequisites</a></li>
        <li><a href="#installation">Installation</a></li>
      </ul>
    </li>
    <li><a href="#usage">Usage</a>
      <ul>
         <li><a href="#qemu">Qemu</a></li>
        <li><a href="#gdb">GDB</a></li>
      </ul>
    </li>
    <li><a href="#roadmap">Roadmap</a></li>
    <li><a href="#references">References</a></li>
    <li><a href="#contact">Contact</a></li>
    <li><a href="#license">License</a></li>
  </ol>
</details>

## About-the-project
![run-qemu](./doc/image/Demo.gif)  
This project is about porting the riscv-xv6 kernel used in MIT teaching to the Loongarch architecture, although there are some xv6 kernel portes in the Loongarch community, but there is no complete kernel porting and porting manual. 
This project will provide a complete kernel and a sufficiently detailed porting manual for those who will participate in the porting work in the future.
<p align="right">(<a href="#readme-top">back to top</a>)</p>

## Docs

+ Kernel migration documentation can be found in [here](./doc/program_doc.pdf)
+ Syscall implementation and test method documentation can be found [here](./doc/test_doc.pdf)
+ Syscall test video can be found [here](https://www.aliyundrive.com/s/XbZ2ZcZtp3E)


## Getting Started
### Prerequisites

To compile the xv6 kernel by LoongArch, you should download the cross-tools

```sh
wget https://github.com/loongson/build-tools/releases/download/2022.05.29/loongarch64-clfs-5.0-cross-tools-gcc-full.tar.xz

sudo tar -vxf loongarch64-clfs-5.0-cross-tools-gcc-full.tar.xz -C /opt
```

Configure environment variables for the cross-tools

```sh
. setenv.sh
```

`setenv.sh`is a script file used to set environment variables

```sh
#!/bin/sh
set -x
CC_PREFIX=/opt/cross-tools

export PATH=$CC_PREFIX/bin:$PATH
export LD_LIBRARY_PATH=$CC_PREFIX/lib:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH=$CC_PREFIX/loongarch64-unknown-linux-gnu/lib/:$LD_LIBRARY_PATH

set +x
```

if your shell is fish, the `setenv.sh` could be

```sh
#!/bin/sh
set CC_PREFIX /opt/cross-tools

fish_add_path $CC_PREFIX/bin
set LD_LIBRARY_PATH $CC_PREFIX/lib $CC_PREFIX/loongarch64-unknown-linux-gnu/lib
```

Run the command to verify whether the setting is successful

```sh
loongarch64-unknown-linux-gnu-gcc --version
```
### Installation
Use `make` to compile kernel and filesystem
```sh
make all
```
You can only complie filesystem
```sh
make fs.img
```

## Usage
### qemu
+ You can simply use `run.sh` to run the kernel

```sh
./run.sh
```

+ Or, You can follow the step below to run

1. First,we need to complie the kernel.The `make all` will compile the filesystem image called `fs.img` ,and runnable binary file called `kernel`
   
   ```sh
   make all
   ```
2. Then cd to `qemu-loongarch-runenv`
   
   ```sh
   ./run_loongarch.sh -k ../kernel/kernel
   ```

### GDB
   1. Install some necessary dependencies and install the required dependencies according to the error situation at compile time.
   ```sh
   sudo apt install texinfo bison flex libgmp-dev
   ```
   2. To compile the gdb tool, it takes a few minutes to compile
   ```sh
   git clone https://github.com/foxsen/binutils-gdb 
   cd inutils-gdb 
   git checkout loongarch-v2022-03-10 
   mkdir build 
   cd build 
   ../configure --target=loongarch64-unknown-linux-gnu --prefix=/opt/gdb 
   make 
   make install
   ```
   3.Starting debugging mode with the -D parameter actually executes the role of debugging server gdbserver, listening to TCP::1234 port by default and waiting for the access of gdb clients.
   ```sh
   ./run_loongarch.sh -k ../kernel/kernel -D
   ```
   4. Start gdb on another terminal and specify the object to be debugged (compiled kernel file path)
   ```sh
   /opt/gdb/bin/loongarch64-unknown-linux-gnu-gdb ../kernel/kernel
   ```
   5.Execute the target remote :1234 command at the gdb command prompt to connect to the xv6 target system
   ```sh
   target remote :1234
   ```
   6.Use the `b` command to set a breakpoint at the function entry of `kernel/main.c`, and use the `c `command to continue executing the xv6 kernel code until the breakpoint is hit(you could use `tui enable` to use gui gdb window)
<p align="right">(<a href="#readme-top">back to top</a>)</p>

### test

+ After starting the kernel successfully, you can use command `ls`, and can see the file
`run-all.sh`

+ you can simply test all the system calls by using
```sh
sh < run-all.sh
```

## Roadmap

- [x] Porting of memory space
- [x] Porting of interrupt exceptions
- [x] Porting of filesystem
- [x] Implement system calls for testing
- [ ] Implement mount system calls
- [ ] Use sd card
- [ ] Run on the 2k500
- [ ] Optimized memory structure 
<p align="right">(<a href="#readme-top">back to top</a>)</p>

## References

+ [xv6-labs-loongarch](https://github.com/Fan33oo/xv6-labs-loongarch)
+ [xv6-riscv-books](https://github.com/FrankZn/xv6-riscv-book-Chinese)
+ [MIT 6.S081-2020-Labs](https://pdos.csail.mit.edu/6.828/2020/xv6.html) 
<p align="right">(<a href="#readme-top">back to top</a>)</p>

## Contact
Pangziwen - pzw2002@mail.ustc.edu.cn

Liqi - liqi123@mail.ustc.edu.cn

Chensiyue - 2723096643@qq.com


Project Link: [https://gitlab.eduxiji.net/202310358111361/project1466467-176097](https://gitlab.eduxiji.net/202310358111361/project1466467-176097)

<p align="right">(<a href="#readme-top">back to top</a>)</p>

## License

[MIT](LICENSE) © Richard Littauer
<p align="right">(<a href="#readme-top">back to top</a>)</p>
