struct buf {
  int valid;   // indicates if the data in the buffer matches the data on disk. It is set to 1 once the buffer has been read from disk.

  int disk;    // indicates if the buffer's data needs to be written to disk. It is set to 1 if the buffer has been modified since being read from disk.

  // dev and blockno specify which disk block this buffer contains.
  uint dev;
  uint blockno;

  // lock is a sleep lock used to lock the buffer for exclusive access. Only one process can lock a buffer at a time.
  struct sleeplock lock;

  // refcnt is a reference count, indicating how many processes are using this buffer. A buffer cannot be evicted from the cache if refcnt > 0.
  uint refcnt;

  // prev and next are pointers used to link the buffer into the LRU list. The LRU list orders buffers from least recently used to most recently used.
  struct buf *prev; // LRU cache list
  struct buf *next;

  // data is a byte array containing the cached disk block data. It is BSIZE (block size) bytes in length.
  uchar data[BSIZE];

  // time is a timestamp indicating when this buffer was last used. It is used to implement LRU eviction of buffers from the cache.
  uint time;   // timeStamp for least recently used (LRU)
};

