	# qemu -kernel loads the kernel at 0x0000000
        # and causes each CPU to jump there.
        # kernel.ld causes the following code to
        # be placed at 0x9000000000000000.
#define		__ASSEMBLY__
#include "memlayout.h"

.section .text
.global _entry
_entry:
	# set up a stack for C.
        # stack0 is declared in start.c,
        # with a 4096-byte stack per CPU.
        # sp = stack0 + (hartid * 4096)

	# This writes 0xa8 to control register 0, setting the privilege level to 0, disabling interrupts, and disabling paging.
	li.w	$t0, 0xa8		# PLV=0, IE=0, PG=0
	csrwr	$t0, 0x0

	# This writes 0x04 to control register 1, keeping the privilege level at 0, enabling interrupts, and disabling write protection.
	li.w	$t0, 0x04		# PLV=0, PIE=1, PWE=0
	csrwr	$t0, 0x1

	# This writes 0x00 to control register 2, disabling various exceptions.
	li.w	$t0, 0x00		# FPE=0, SXE=0, ASXE=0, BTE=0
	csrwr	$t0, 0x2

	# 使用映射窗口映射整个物理内存
	# 通过模拟一个tlb例外来实现
	li.d	$t0, CSR_DMW1_INIT
	csrwr	$t0, 0x181
	
	# 设置例外前模式
	li.w    $t0, 0x00		# PPLV=0, PPIE=0, PPWE=0
	csrwr	$t0, 0x8b

	# 计算ertn后一条指令的虚拟地址
	li.d	$t0, CSR_DMW1_BASE
	pcaddi	$t1, 0x5		    # ------+
	add.d	$t0, $t0, $t1	    #       |
                                #       |
    # 将虚拟地址写入tlbrera        #      |  
	# 并设置tlbrera.IsTLBR	     #       |
	ori	$t0, $t0, 0x01		    #	    |
	csrwr	$t0, 0x8a		    #       |
	ertn				        #       |
                                #	    |
	la.abs	$sp, stack0+4096	# <-----+
	csrrd	$t0, 0x20		    # CPUID
	andi	$t0, $t0, 0x3ff
	slli.d	$t0, $t0, 12
	add.d	$sp, $sp, $t0
    # 实现每个CPU拥有独立的栈空间。
    # 通过使用CPUID作为偏移量，可以为每个CPU分配一个唯一的栈空间，避免不同CPU之间的栈数据互相干扰。

    bl main				# call main in main.c
spin:
	b spin